@@ -33,7 +33,9 @@ merge.xts <- function(...,
   mc <- match.call(expand=FALSE)
   dots <- mc$...
   if(is.null(suffixes)) {
-    syms <- as.character(dots)
+    syms <- names(dots)
+    syms[nchar(syms)==0] <- as.character(dots)[nchar(syms)==0]
+    if(is.null(syms)) syms <- as.character(dots)
   } else
   if(length(suffixes) != length(dots)) {
     warning("length of suffixes and does not match number of merged objects")
